/*
 *	SCCS: @(#)tet_api.h	1.22 (96/10/21)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1996 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 * A copy of the end-user licence agreement is contained in the file
 * Licence which accompanies this distribution.
 * 
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

/*
 * Copyright 1990 Open Software Foundation (OSF)
 * Copyright 1990 Unix International (UI)
 * Copyright 1990 X/Open Company Limited (X/Open)
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of OSF, UI or X/Open not be used in 
 * advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission.  OSF, UI and X/Open make 
 * no representations about the suitability of this software for any purpose.  
 * It is provided "as is" without express or implied warranty.
 *
 * OSF, UI and X/Open DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO 
 * EVENT SHALL OSF, UI or X/Open BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
 * PERFORMANCE OF THIS SOFTWARE.
 */

/************************************************************************

SCCS:   	@(#)tet_api.h	1.22 96/10/21 TETware release 3.1
NAME:		'C' API header
PRODUCT:	TETware
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 July 1990
CONTENTS:

	struct tet_testlist definition
	result code values for use with tet_result()
	TET_NULLFP null function pointer for use with tet_fork()
	declarations/prototypes for all API interfaces

MODIFICATIONS:

	Geoff Clare, UniSoft Ltd., 10 Oct 1990
		Remove const keywords.

	Geoff Clare, UniSoft Ltd., 18 Oct 1990
		Add tet_pname.

	Geoff Clare, UniSoft Ltd., 28 Nov 1990
		Add tet_nosigreset.

	Geoff Clare, UniSoft Ltd., 21 June 1991
		Make tet_fork() prototype consistent with definition.

	DTET development - this file is derived from TET Release 1.10
	David G. Sawyer
	John-Paul Leyland
	UniSoft Ltd, June 1992

	Andrew Dingwall, UniSoft Ltd., October 1992
	Added DTET API function declarations.

	Denis McConalogue, UniSoft Limited, September 1993
	changed prototype for tet_sync()

	Andrew Dingwall, UniSoft Ltd., February 1994
	removed comment from #include line (for strict ANSI compliance?)

	Andrew Dingwall, UniSoft Ltd., October 1994
	added support for tet_msync() API function

	Geoff Clare, UniSoft Ltd., July 1996
	Changes for TETWare.

	Geoff Clare, UniSoft Ltd., Sept 1996
	Changes for TETWare-Lite and POSIX threads.

************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/* the following are needed for types used in declarations */
#include <sys/types.h>
#ifdef _WIN32
typedef int pid_t;
#endif
#if defined(__STDC__) || defined(__cplusplus)
#include <time.h>
#include <stdarg.h>
#endif
#ifdef TET_POSIX_THREADS
#include <pthread.h>
#else /* !TET_POSIX_THREADS */
#  ifdef TET_THREADS
#  include <thread.h>
#  endif
#endif /* !TET_POSIX_THREADS */

/* argument values for tet_result() */

#define TET_PASS	0
#define TET_FAIL	1
#define TET_UNRESOLVED	2
#define TET_NOTINUSE	3
#define TET_UNSUPPORTED	4
#define TET_UNTESTED	5
#define TET_UNINITIATED	6
#define TET_NORESULT	7

/* tet_errno values, used to index into tet_errlist[] */

#define TET_ER_OK		0	/* ok  success */
#define TET_ER_ERR		1	/* general error code */
#define TET_ER_MAGIC		2	/* bad magic number */
#define TET_ER_LOGON		3	/* not logged on */
#define TET_ER_RCVERR		4	/* receive message error */
#define TET_ER_REQ		5	/* unknown request code */
#define TET_ER_TIMEDOUT		6	/* request timed out */
#define TET_ER_DUPS		7	/* request contained duplicate IDs */
#define TET_ER_SYNCERR		8	/* sync completed unsuccessfully */
#define TET_ER_INVAL		9	/* invalid request parameter */
#define TET_ER_TRACE		10	/* tracing not configured */
#define TET_ER_WAIT		11	/* process not terminated */
#define TET_ER_XRID		12	/* bad xrid in xresd request */
#define TET_ER_SNID		13	/* bad snid in syncd request */
#define TET_ER_SYSID		14	/* sysid not in system name list */
#define TET_ER_INPROGRESS	15	/* event in progress */
#define TET_ER_DONE		16	/* event finished or already happened */
#define TET_ER_CONTEXT		17	/* request out of context */
#define TET_ER_PERM		18	/* priv request/kill error */
#define TET_ER_FORK		19	/* can't fork */
#define TET_ER_NOENT		20	/* no such file or directory */
#define TET_ER_PID		21	/* no such process */
#define TET_ER_SIGNUM		22	/* bad signal number */
#define TET_ER_FID		23	/* bad file id */
#define TET_ER_INTERN		24	/* server internal error */
#define TET_ER_ABORT		25	/* abort TCM on TP end */
#define TET_ER_2BIG		26	/* argument list too long */

#define TET_NULLFP	((void (*)())0)	/* for use as argument to tet_fork() */

#ifndef TET_LITE /* -START-LITE-CUT- */

/* structure used in tet_msync() and tet_remsync() calls */
struct tet_synmsg {
	char *tsm_data;	/* ptr to sync message data buffer */
	int tsm_dlen;	/* no of bytes in sm_data */
	int tsm_sysid;	/* id of system sending sync message data */
	int tsm_flags;	/* flags - see below */
};

/* values for tsm_flags (a bit field) */
#define TET_SMSNDMSG	001	/* system is sending message data */
#define TET_SMRCVMSG	002	/* system is receiving message data */
#define TET_SMDUP	004	/* more than one system attempted to send */
#define TET_SMTRUNC	010	/* message data was truncated */

#define TET_SMMSGMAX	1024	/* maximum size of a tet_msync message -
					size must be expressable in 12 bits
					(see ST_COUNTMASK in ldst.h) */

/* definitions for use with tet_getsysbyid() */

#define TET_SNAMELEN	32	/* maximum system name length */

struct tet_sysent {
	int ts_sysid;			/* system ID */
	char ts_name[TET_SNAMELEN];	/* system name */
};

/* sync requests and error reporting */

/* sync votes */
#define TET_SV_YES		1
#define TET_SV_NO		2

struct tet_syncstat {
	int tsy_sysid;	/* system ID */
	int tsy_state;	/* sync state */
};

/* values for tsy_state */
#define TET_SS_NOTSYNCED	1	/* sync request not received */
#define TET_SS_SYNCYES		2	/* system voted YES */
#define TET_SS_SYNCNO		3	/* system voted NO */
#define TET_SS_TIMEDOUT		4	/* system timed out */
#define TET_SS_DEAD		5	/* process exited */

#endif /* -END-LITE-CUT- */

/* API functions and data */

#if defined(__STDC__) || defined(__cplusplus)

struct tet_testlist {
	void (*testfunc)(void);
	int icref;
};

extern void	tet_delete(int, char *);
extern int	tet_exec(char *, char *[], char *[]);
extern void	tet_exit(int);
extern int	tet_fork(void (*)(), void (*)(), int, int);
extern char *	tet_getvar(char *);
extern void	tet_infoline(char *);
extern int	tet_kill(pid_t, int);
extern void	tet_logoff(void);
extern int	tet_minfoline(char **, int);
extern int	tet_printf(char *, ...);
extern char *	tet_reason(int);
extern int	tet_remgetlist(int **);
extern int	tet_remgetsys(void);
extern void	tet_result(int);
extern void	tet_setblock(void);
extern void	tet_setcontext(void);
extern pid_t	tet_spawn(char *, char **, char **);
extern int	tet_vprintf(char *, va_list);
extern int	tet_wait(pid_t, int *);

#ifndef TET_LITE /* -START-LITE-CUT- */
extern int	tet_getsysbyid(int, struct tet_sysent *);
extern int	tet_msync(long, int *, int, struct tet_synmsg *);
extern int	tet_remexec(int, char *, char **);
extern int	tet_remkill(int);
extern int	tet_remsync(long, int *, int, int, int, struct tet_synmsg *);
extern int	tet_remtime(int, time_t *);
extern int	tet_remwait(int, int, int *);
extern int	tet_sync(long, int *, int);
extern void	(*tet_syncerr)(long, struct tet_syncstat *, int);
extern void	tet_syncreport(long, struct tet_syncstat *, int);
#endif /* -END-LITE-CUT- */

#if defined(TET_THREADS) || defined(TET_POSIX_THREADS)
extern int	tet_fork1(void (*)(), void (*)(), int, int);
extern pid_t *	tet_thr_child(void);
#  ifdef TET_POSIX_THREADS
extern int	tet_pthread_create(pthread_t *, pthread_attr_t *, void *(*)(void *), void *, int);
#  else /* !TET_POSIX_THREADS */
extern int	tet_thr_create(void *, size_t, void *(*)(void *), void *, long, thread_t *, int);
#  endif /* !TET_POSIX_THREADS */
extern int *	tet_thr_errno(void);
#endif /* THREADS */

#else /* !__STDC__ */

struct tet_testlist {
	void (*testfunc)();
	int icref;
};

extern void	tet_delete();
extern int	tet_exec();
extern void	tet_exit();
extern int	tet_fork();
extern char *	tet_getvar();
extern void	tet_infoline();
extern int	tet_kill();
extern void	tet_logoff();
extern int	tet_minfoline();
extern int	tet_printf();
extern char *	tet_reason();
extern int	tet_remgetlist();
extern int	tet_remgetsys();
extern void	tet_result();
extern void	tet_setblock();
extern void	tet_setcontext();
extern pid_t	tet_spawn();
extern int	tet_vprintf();
extern int	tet_wait();

#ifndef TET_LITE /* -START-LITE-CUT- */
extern int	tet_getsysbyid();
extern int	tet_msync();
extern int	tet_remexec();
extern int	tet_remkill();
extern int	tet_remsync();
extern int	tet_remtime();
extern int	tet_remwait();
extern int	tet_sync();
extern void	(*tet_syncerr)();
extern void	tet_syncreport();
#endif /* -END-LITE-CUT- */

#if defined(TET_THREADS) || defined(TET_POSIX_THREADS)
extern int	tet_fork1();
extern pid_t *	tet_thr_child();
#  ifdef TET_POSIX_THREADS
extern int	tet_pthread_create();
#  else /* !TET_POSIX_THREADS */
extern int	tet_thr_create();
#  endif /* !TET_POSIX_THREADS */
extern int *	tet_thr_errno();
#endif /* THREADS */

#endif /* !__STDC__ */

#if defined(TET_THREADS) || defined(TET_POSIX_THREADS)
#define tet_child (*tet_thr_child())
#define tet_errno (*tet_thr_errno())
#else /* !THREADS */
extern pid_t	tet_child;
extern int	tet_errno;
#endif /* !THREADS */

extern char *	tet_errlist[];
extern int	tet_nerr;
extern int	tet_nosigreset;
extern char *	tet_pname;
extern int	tet_thistest;

#ifdef __cplusplus
}
#endif
