/*
 *	SCCS: @(#)dtcm.c	1.25 (97/03/27)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1996 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 * A copy of the end-user licence agreement is contained in the file
 * Licence which accompanies this distribution.
 * 
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

/*
 * Copyright 1990 Open Software Foundation (OSF)
 * Copyright 1990 Unix International (UI)
 * Copyright 1990 X/Open Company Limited (X/Open)
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of OSF, UI or X/Open not be used in 
 * advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission.  OSF, UI and X/Open make 
 * no representations about the suitability of this software for any purpose.  
 * It is provided "as is" without express or implied warranty.
 *
 * OSF, UI and X/Open DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO 
 * EVENT SHALL OSF, UI or X/Open BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR 
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef lint
static char sccsid[] = "@(#)dtcm.c	1.25 (97/03/27) TETware release 3.1";
#endif

/************************************************************************

SCCS:   	@(#)dtcm.c	1.25 97/03/27 TETware release 3.1
NAME:		'C' Test Case Manager
PRODUCT:	TETware
AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 July 1990
SYNOPSIS:

	int tet_tcm_main(int argc, char **argv);
	int tet_thistest;

DESCRIPTION:

	Tcm_main() is the main program for the Test Case Manager (TCM),
	called from a different main() wrapper for each API.
	The command line contains the invocable components (IC's) to be
	executed.  All test purposes in the user-supplied tet_testlist[]
	array with an icref element corresponding to one of the requested
	IC's are called, preceded by a call to the function pointed to
	by tet_startup and followed by tet_cleanup (if each is not NULL).
	The TCM reads configuration variables from the file specified
	by the environment variable TET_CONFIG, provides handling of
	unexpected signals on UNIX-like platforms, and reports the start
	and end of IC's and the start of test purposes.

	Tet_thistest is set to the test purpose number during execution
	of each test purpose.  Test purposes are numbered in the sequence
	of the tet_testlist[] array (starting at 1) regardless of
	execution sequence.

MODIFICATIONS:

	Geoff Clare, UniSoft Ltd, 6 Sept 1990
		New results file format.
		If no arguments assume all ICs.
		Move tet_error() to resfile.c.
		Change handling of ICs with zero TPs.

	Geoff Clare, UniSoft Ltd, 28 Sept 1990
		When doing all ICs start from lowest defined IC instead of 1.
	
	Geoff Clare, UniSoft Ltd, 28 Nov 1990
		Add tet_nosigreset.
	
	Geoff Clare, UniSoft Ltd, 6 Sept 1991
		Add call to tet_delreas().

	DTET development - this file is based on release 1.10 of the TET
	David G. Sawyer and John-Paul Leyland, UniSoft Ltd, June 1992

	Andrew Dingwall, UniSoft Ltd., October 1992
	Moved tet_root[] and setup to tcmfuncs.c so that tcmchild
	processes can see it as well.

	Updated Abort, Delete and results processing to work in a DTET
	environment.

	Andrew Dingwall, UniSoft Ltd., February 1995
	added support for the UniSoft TCL API
	(this API is not part of the TETware distribution)

	Geoff Clare, UniSoft Ltd, July-August 1996
	Changes for TETWare.

	Andrew Dingwall, UniSoft Ltd., August 1996
	changes for use with tetware tcc

	Geoff Clare, UniSoft Ltd, Sept 1996
	Added call to tet_init_blockable_sigs().
	Changes for TETWare-Lite.

	Geoff Clare, UniSoft Ltd., Oct 1996
	Use tet_thr_equal() to compare thread IDs.
	Restructured tcm source to avoid "ld -r".

************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#  include <unistd.h>
#  include <setjmp.h>
#  include <signal.h>
#include <string.h>
#include <errno.h>
#include "dtmac.h"
#include "tet_api.h"
#include "error.h"
#include "dtmsg.h"
#include "dtthr.h"
#include "alarm.h"
#include "synreq.h"
#include "tcmfuncs.h"
#include "dtetlib.h"

#ifdef TET_LITE /* -LITE-CUT-LINE- */
#  define	VERSION		"3.1-lite"
#else /* -START-LITE-CUT- */
#  define	VERSION		"3.1"
#endif /* -END-LITE-CUT- */
#define ARGS_COUNT	0
#define ARGS_EXECUTE	1
#define	KILLWAIT	10

extern	void	(*tet_startup)();
extern	void	(*tet_cleanup)();

#ifdef TCLAPI
extern	struct tet_testlist *tet_testlist;
#else
extern	struct tet_testlist tet_testlist[];
#endif

#ifndef TET_LITE /* -START-LITE-CUT- */
extern	int	tet_sderrno;
extern	int	tet_sync_del;
extern	struct synreq *tet_synreq;
#else /* -END-LITE-CUT- */
extern	char *	tet_tmpresfile;
extern	void	tet_openres();
#endif /* -LITE-CUT-LINE- */
extern	void	tet_config();
extern	void	tet_delreas();
extern	void	tet_error();
#ifndef _WIN32
extern	int	tet_killw();
extern sigset_t tet_blockable_sigs;
#endif /* !_WIN32 */
extern	char *	tet_signame();
extern	long	tet_activity;
#ifdef TET_THREADS
extern thread_t tet_start_tid;
extern	long	tet_next_block;
extern	void	tet_cln_threads();
extern	void	tet_mtx_init();
extern	void	tet_mtx_destroy();
#endif /* TET_THREADS */
extern	int	tet_Nsname;
extern	char *	tet_pname;
extern	int 	tet_iclast;


static	pid_t	toppid;

#ifndef _WIN32
static	int	sigreset = 1;
static	sigjmp_buf skipjmp;
#endif /* !_WIN32 */

#ifdef TET_THREADS
static	int	start_errno;
static	int	start_child;
static	int	start_alrm_flag;
static	long	start_block;
static	long	start_sequence;
#endif /* TET_THREADS */

static	int	procargs();
static	int	count_tps();
static	int	callfuncs();

#ifndef _WIN32
static	void	sigskip();
static	void	sigabandon();
static	void	sigterm();
static	void	setsigs();
#endif /* !_WIN32 */

#ifndef TET_LITE /* -START-LITE-CUT- */
static	void	sync_report();
static	void	init_synreq();
#endif /* -END-LITE-CUT- */

static	char	buf[256];

/* ARGSUSED */
int
tet_tcm_main(argc, argv)
int	argc;
char	**argv;
{
	extern int tet_myptype, tet_combined_ok;
	char *cp;
	int nsys;

	tet_pname = argv[0]; /* used by tet_error() */

	/* save current PID for checking when unexpected signals
	   cause a longjmp() to main test loop */
	toppid = GETPID();

#ifdef TET_THREADS
	/* save initial thread ID for checking on unexpected signals */
	tet_start_tid = thr_self();

	/* set up thread-specific data */
	if (thr_keycreate(&tet_errno_key, TET_NULLFP) != 0 ||
	    thr_keycreate(&tet_child_key, TET_NULLFP) != 0 ||
	    thr_keycreate(&tet_alrm_flag_key, TET_NULLFP) != 0 ||
	    thr_keycreate(&tet_block_key, TET_NULLFP) != 0 ||
	    thr_keycreate(&tet_sequence_key, TET_NULLFP) != 0)
	{
		/* can't use tet_error() yet */
		(void) fprintf(stderr,
			"%s: thr_keycreate() failed in TCM startup",
			tet_pname);
		exit(EXIT_FAILURE);
	}
	(void) thr_setspecific(tet_errno_key, (void *)&start_errno);
	(void) thr_setspecific(tet_child_key, (void *)&start_child);
	(void) thr_setspecific(tet_alrm_flag_key, (void *)&start_alrm_flag);
	(void) thr_setspecific(tet_block_key, (void *)&start_block);
	(void) thr_setspecific(tet_sequence_key, (void *)&start_sequence);

	/* initialise all mutexes */
	tet_mtx_init();
#endif

	/* blockable signals reported "unexpected" if caught (updated later) */
	tet_init_blockable_sigs();

	/* set tet_activity from environment */
	cp = getenv("TET_ACTIVITY");
	if (cp == NULL || *cp == '\0')
		tet_activity = 0;
	else
		tet_activity = atol(cp);

#ifdef TET_LITE
	/* open execution results file (do early, so tet_error() can use it) */
	tet_openres();
#endif /* TET_LITE */

	/* initialise the server and transport stuff */
	tet_tcminit(argc, argv);

	/* read in configuration variables */
	tet_config();

	/* set up data for use by tet_delete() and tet_reason() */
	tet_delreas(count_tps(-1));

	/* set current context to process ID */
	tet_setcontext();

	/* count number of IC's to be executed and output on TCM Start line */
	tet_tcmstart(VERSION, procargs(argv, ARGS_COUNT));

#ifndef _WIN32
	/* unexpected signals are fatal during startup */
	setsigs(sigabandon);
#endif /* !_WIN32 */

#ifndef TET_LITE /* -START-LITE-CUT- */
	/* Do an auto sync -> use ic = 0, tp = 1 */
	init_synreq();
	nsys = tet_Nsname;
	if (tet_tcm_async(MK_ASPNO(0, 1, S_ICSTART), SV_YES, SV_SYNC_TIMEOUT,
		tet_synreq, &nsys) < 0) {
			tet_error(tet_sderrno,
				"startup function Auto Sync failed");
			tet_exit(EXIT_FAILURE);
	}

	/* now, analyse the results of an unsuccessful auto sync */
	if (tet_sderrno != ER_OK) {
		sync_report(nsys, "startup");
		tet_exit(EXIT_FAILURE);
	}
#endif /* -END-LITE-CUT- */

	/* call user-supplied startup function */
	if (tet_startup != NULL)
	{
		(*tet_startup)();
#ifdef TET_THREADS
		tet_cln_threads(0);
		tet_mtx_destroy();
		tet_mtx_init();
#endif
	}
	
#ifndef _WIN32
	/* see whether tet_nosigreset was set in startup function */
	if (tet_nosigreset)
		sigreset = 0;
#endif
	
	/* execute IC's specified on command line */
	(void) procargs(argv, ARGS_EXECUTE);

#ifndef _WIN32
	/* unexpected signals are fatal during cleanup */
	setsigs(sigabandon);
#endif /* !_WIN32 */

	/* call the user-supplied cleanup routine (if any) */
	tet_docleanup();

	/* log of all the servers and exit */
	tet_exit(EXIT_SUCCESS);

	/* NOTREACHED */
}

static int
procargs(argv, action)
char **argv;
int action;
{
	/*
	 * Process the IC list given on the command line.  If action is
	 * ARGS_COUNT simply return an IC count.  If action is ARGS_EXECUTE
	 * then execute each IC.
	 */

	char *cp;
	struct tet_testlist *tp;
	int ic_no, ic_first, ic_count, testcnt;
	static char *default_args[] = { "arg0", "all", NULL };

	/*
	 * Each argument contains an invocable component list of the
	 * form: 1,3-5,7,11,... 
	 *
	 * If any argument is "all" then all components later than
	 * the last in the previous argument are executed.
	 *
	 * Default if no arguments are given is "all".
	 */

	if (argv[1] == NULL)
		argv = default_args;

	tet_iclast = -1;
	ic_count = 0;
	while (*++argv != NULL)
	{
		if (strncmp(*argv, "all", (size_t)3) == 0)
		{
			if (tet_iclast < 0)
			{
				/* no previous ICs executed - start at lowest */
				ic_first = tet_testlist[0].icref;
				for (tp = tet_testlist;
						tp->testfunc != NULL; tp++)
				{
					if (tp->icref < ic_first)
						ic_first = tp->icref;
				}
			}
			else
				/* carry on from last IC executed */
				ic_first = tet_iclast + 1;

			tet_iclast = ic_first;
			for (tp = tet_testlist; tp->testfunc != NULL; tp++)
			{
				if (tp->icref > tet_iclast)
					tet_iclast = tp->icref;
			}

			for (ic_no = ic_first; ic_no <= tet_iclast; ic_no++)
			{
				testcnt = count_tps(ic_no);

				/* skip ICs with zero TPs */
				if (testcnt == 0)
					continue;

				ic_count++;
				if (action == ARGS_EXECUTE)
				{
					/* output IC start message */
					tet_icstart(ic_no, testcnt);

					testcnt = callfuncs(ic_no);

					/* output IC end message */
					tet_icend(ic_no, testcnt);
				}
			}
		}
		else
		{
			for (cp = *argv; *cp != '\0'; cp++)
			{
				ic_first = atoi(cp);
				while(*cp != ',' && *cp != '-' && *cp != '\0')
					cp++;
				if (*cp == '-')
				{
					tet_iclast = atoi(++cp);
					while(*cp != ',' && *cp != '\0')
						cp++;
				}
				else
					tet_iclast = ic_first;

				for (ic_no = ic_first;
						ic_no <= tet_iclast; ic_no++)
				{
					testcnt = count_tps(ic_no);

					/* skip ICs with zero TPs */
					if (testcnt == 0)
					{
						if (action == ARGS_EXECUTE &&
						   (ic_no == ic_first ||
						    ic_no == tet_iclast))
						{
							(void) sprintf(buf,
				    "IC %d is not defined for this test case",
								ic_no);
							tet_error(0, buf);
						}
						continue;
					}

					ic_count++;
					if (action == ARGS_EXECUTE)
					{
						/* output IC start message */
						tet_icstart(ic_no, testcnt);

						testcnt = callfuncs(ic_no);

						/* output IC end message */
						tet_icend(ic_no, testcnt);
					}
				}

				if (*cp == '\0')
					break;
			}
		}
	}

	return ic_count;
}

static int
count_tps(ic_no)
int ic_no;
{
	/* count number of tests in specified invocable component */
	/* or in all ICs if ic_no < 0 */

	struct tet_testlist *tp;
	int testcnt = 0;

	for (tp = tet_testlist; tp->testfunc != NULL; tp++)
	{
		if (ic_no < 0 || tp->icref == ic_no)
			testcnt++;
	}

	return testcnt;
}

static int
callfuncs(ic_no)
int ic_no;
{
	/* call each test function in specified invocable component */
	/* return count of test functions called */

	struct tet_testlist *tp;
	int test_no, testcnt;
	int signum;

	testcnt = 0;
	for (test_no = 1, tp = tet_testlist;
			tp->testfunc != NULL; test_no++, tp++)
	{
		if (tp->icref != ic_no)
			continue;
#ifndef _WIN32	/* no signal handling in WIN32 */
		else if ((signum = sigsetjmp(skipjmp, 1)) != 0)
		{
			(void) sprintf(buf,
				"unexpected signal %d (%s) received",
				signum-1, tet_signame(signum-1));
			tet_infoline(buf);
			tet_result(TET_UNRESOLVED);
			if (tet_child > 0)
			{
				(void) tet_killw(tet_child, KILLWAIT);
				tet_child = 0;
			}
#ifdef TET_THREADS
			tet_cln_threads(signum-1);
			tet_mtx_destroy();
			tet_mtx_init();
#endif /* TET_THREADS */

			/* if this is not the top level process, don't fall
			   through to TCM test purpose loop! */
			if (GETPID() != toppid)
				exit(EXIT_FAILURE);

			if (signum-1 == SIGTERM)
				sigterm();

			/* output unresolved result and go on to next test */
			tet_tpend(ic_no, test_no);
			testcnt++;
			continue;
		}
#endif /* !_WIN32 */

		/* output test start message */
		tet_tpstart(ic_no, test_no);

		/* set global current test purpose indicator */
		tet_thistest = test_no;

		if (tet_reason(test_no) != NULL)
		{
			/* TP deleted in this TCM */
			tet_infoline(tet_reason(test_no));
			tet_result(TET_UNINITIATED);
		}
#ifndef TET_LITE /* -START-LITE-CUT- */
		else if (tet_sync_del != 0)
		{
			/* TP deleted in another TCM */
			tet_result(TET_UNINITIATED);
			tet_sync_del = 0;
		}
#endif /* -END-LITE-CUT- */
		else
		{

#ifndef _WIN32
			/* unexpected signals skip current test
			   NOTE: this is done before every test function
			   to ensure no "local" signal handlers are left
			   in place when skipping to the next test.
			   This safety feature can be disabled by setting
			   tet_nosigreset in the tet_startup function,
			   in which case unexpected signals in later
			   tests could go unnoticed. */
			if (sigreset)
				setsigs(sigskip);
#endif /* !_WIN32 */

			/* call the user-supplied test function */
			(*tp->testfunc)();
#ifdef TET_THREADS
			tet_cln_threads(0);
			tet_mtx_destroy();
			tet_mtx_init();
#endif
		}

		/* output test result */
		tet_tpend(ic_no, test_no);

		testcnt++;
	}

	return testcnt;
}


#ifndef _WIN32

static	void
sigabandon(sig)
int	sig;
{
	static char	mbuf[132];

	if (sig == SIGTERM)
		sigterm();

#ifdef TET_THREADS
	if (!tet_thr_equal(thr_self(), tet_start_tid))
	{
		(void) thr_kill(tet_start_tid, sig);
		thr_exit((void *)0);
	}
#endif /* TET_THREADS */

	(void) sprintf(mbuf,
		"Abandoning testset: caught unexpected signal %d (%s)",
		sig, tet_signame(sig));
	tet_error(0, mbuf);

#ifdef TET_LITE
	if (tet_tmpresfile != NULL)
		(void) UNLINK(tet_tmpresfile);
#endif /* TET_LITE */

	/* log off all the servers and exit */
	tet_exit(EXIT_FAILURE);
}

static	void
sigterm()
{
	/*  Cleanup and exit if SIGTERM received */

	char *msg = "Abandoning test case: received signal SIGTERM";

	/* terminate [per-thread] child, if any */
	if (tet_child > 0)
	{
		(void) tet_killw(tet_child, KILLWAIT);
		tet_child = 0;
	}

#ifdef TET_THREADS
	if (!tet_thr_equal(thr_self(), tet_start_tid))
	{
		(void) thr_kill(tet_start_tid, SIGTERM);
		thr_exit((void *)0);
	}

	/* only main thread gets to here */
	tet_cln_threads(SIGTERM);
#endif /* TET_THREADS */

	tet_error(0, msg);

#ifdef TET_LITE
	if (tet_tmpresfile != NULL)
		(void) UNLINK(tet_tmpresfile);
#endif /* TET_LITE */

	/* call user-supplied cleanup function */
	if (tet_cleanup != NULL)
	{
		tet_thistest = 0;
		(*tet_cleanup)();
#ifdef TET_THREADS
		tet_cln_threads(0);
		tet_mtx_destroy();
		tet_mtx_init();  /* needed by tet_exit() */
#endif /* TET_THREADS */
	}

	/* log off all the servers and exit */
	tet_exit(EXIT_FAILURE);
}


static	void
sigskip(sig)
int sig;
{
	/*
	 * Catch unexpected signals and (in main thread) longjmp() to
	 * skipjmp where the setjmp() return value will be (sig+1).
	 */

#ifdef TET_THREADS
	if (!tet_thr_equal(thr_self(), tet_start_tid))
	{
		(void) thr_kill(tet_start_tid, sig);
		thr_exit((void *)0);
	}
#endif /* TET_THREADS */

	siglongjmp(skipjmp, sig+1);
}


static void
sig_init(var, set)
char *var;
sigset_t *set;
{
	/* initialise signal set from list in specified variable */

	/* note that this routine uses strtok() which will alter the
	   contents of the list variable */

	char	*list, *sname;
	int	snum;

	(void) sigemptyset(set);

	list = tet_getvar(var);
	if (list == NULL)
		return;

	for (sname = strtok(list, ", "); sname != NULL;
					sname = strtok((char *) NULL, ", "))
	{
		snum = atoi(sname);

		/* Check it's not a standard signal */
		if (strncmp(tet_signame(snum), "SIG", (size_t)3) == 0)
		{
			(void) sprintf(buf,
			    "warning: illegal entry \"%s\" in %s ignored",
			    sname, var);
			tet_error(0, buf);
		}
		else if (sigaddset(set, snum) == -1)
		{
			(void) sprintf(buf,
			    "warning: sigaddset() failed on entry \"%s\" in %s",
			    sname, var);
			tet_error(0, buf);
		}
	}
}


static	void
setsigs(func)
void	(*func)();
{
	/*
	 * Sets all signals except SIGKILL, SIGSTOP and SIGCHLD
	 * to be caught by "func", except that signals specified in
	 * TET_SIG_IGN are ignored and signals specified in
	 * TET_SIG_LEAVE are left alone.
	 */

	int i;
	struct sigaction sig;
	static sigset_t	sig_ign;
	static sigset_t	sig_leave;
	static int	init_done = 0;

	if (!init_done)
	{
		sig_init("TET_SIG_IGN", &sig_ign);
		sig_init("TET_SIG_LEAVE", &sig_leave);
		init_done = 1;
	}

	(void) sigemptyset(&tet_blockable_sigs);

	/* NSIG is not provided by POSIX.1:  it must be defined via
	   an extra feature-test macro, or on the compiler command line */
	for (i = 1; i < NSIG; i++)
	{
		if (i == SIGKILL || i == SIGSTOP || i == SIGCHLD
					|| sigismember(&sig_leave, i))
			continue;
		
		if (sigismember(&sig_ign, i))
			sig.sa_handler = SIG_IGN;
		else
			sig.sa_handler = func;
		sig.sa_flags = 0;
		(void) sigemptyset(&sig.sa_mask);
		if (sigaction(i, &sig, (struct sigaction *)NULL) == 0 &&
#ifdef SIGBUS
		    i != SIGBUS &&
#endif
		    i != SIGSEGV && i != SIGILL && i != SIGFPE)
			(void) sigaddset(&tet_blockable_sigs, i);
	}
}

#endif /* !_WIN32 */


/*
**	tet_docleanup() - call the user-supplied cleanup function (if any)
*/

void tet_docleanup()
{
#ifndef TET_LITE /* -START-LITE-CUT- */
	int nsys;

	/* Do an auto sync -> use ic = tet_iclast + 1, tp = 1 */
	nsys = tet_Nsname;
	if (tet_tcm_async(MK_ASPNO(tet_iclast + 1, 1, S_TPSTART), SV_YES,
		SV_SYNC_TIMEOUT, tet_synreq, &nsys) < 0) {
			tet_error(tet_sderrno,
				"cleanup function Auto Sync failed");
			tet_exit(EXIT_FAILURE);
	}

	/* analyse the results of an unsuccessful auto sync */
	if (tet_sderrno != ER_OK) {
		sync_report(nsys, "cleanup");
		tet_exit(EXIT_FAILURE);
	}
#endif /* -END-LITE-CUT- */

	/* call user-supplied cleanup function */
	if (tet_cleanup != NULL)
	{
		tet_thistest = 0;
#ifndef TET_THREADS
		tet_block = 0;
#else
		tet_next_block = 0;
#endif
		tet_setblock();

		(*tet_cleanup)();

#ifdef TET_THREADS
		tet_cln_threads(0);
		tet_mtx_destroy();
		tet_mtx_init();
#endif
	}
}

#ifndef TET_LITE /* -START-LITE-CUT- */

/*
**	init_synreq() - allocate memory for the tet_synreq array
*/

static void init_synreq()
{
	if (tet_synreq != (struct synreq *) 0)
		return;

	errno = 0;
	if ((tet_synreq = (struct synreq *) malloc(sizeof *tet_synreq * tet_Nsname)) == (struct synreq *) 0) {
		tet_error(errno, "can't allocate memory for tet_synreq array");
		tet_exit(EXIT_FAILURE);
	}
}

/*
**	sync_report() - analyse the results of the startup and cleanup
**		tet_tcm_async() calls
*/

static void sync_report(nsys, functype)
register int nsys;
char *functype;
{
	register struct synreq *sp;
	char errmsg[128];

	for (sp = tet_synreq; sp < tet_synreq + nsys; sp++)
		switch (sp->sy_state) {
		case SS_SYNCYES:
			break;
		default:
			(void) sprintf(errmsg,
		"%s function Auto Sync error, sysid = %d, state = %s",
				functype, sp->sy_sysid,
				tet_systate(sp->sy_state));
			tet_error(tet_sderrno, errmsg);
			break;
		}
}

#endif /* -END-LITE-CUT- */
